/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.privacy;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Randoms;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponentFactory;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

public class PrivacyComponentFactory implements HostComponentFactory {
	private List<PrivacyLevelFrequency> _levels = new ArrayList<>();
	private double _sum = 0;
	private Random _random = Randoms.getRandom(getClass());

	private double[] _ratio;

	@Override
	public HostComponent createComponent(Host pHost) {
		PrivacyLevel level = PrivacyLevel.NO_PRIVACY;

		double randomValue = _random.nextDouble() * _sum;
		double count = 0;
		for (int i = 0; i < _levels.size(); i++) {
			count += _levels.get(i).getProbability();
			if (count > randomValue) {
				level = _levels.get(i).getLevel();
				break;
			}
		}
		PrivacyComponent privacyComponent = new PrivacyComponent(pHost);
		privacyComponent.setPrivacy(level);
		return privacyComponent;
	}

	public void setRatio(String pRatio) {
		String[] split = pRatio.split("-");
		_ratio = new double[split.length];
		for (int i = 0; i < split.length; i++) {
			_ratio[i] = Double.valueOf(split[i]);
			_sum += _ratio[i];
		}
	}

	public void setPrivacy(PrivacyLevel pPrivacyLevel) {
		_levels.add(new PrivacyLevelFrequency(pPrivacyLevel, _ratio[_levels.size()]));

		PrivacyLevel.PRIVACY_LEVELS.add(pPrivacyLevel);
	}

	public static class PrivacyLevelFrequency {
		private PrivacyLevel _level;
		private double _probability;

		@XMLConfigurableConstructor({ "privacy", "probability" })
		public PrivacyLevelFrequency(PrivacyLevel pLevel, double pProbability) {
			_level = pLevel;
			_probability = pProbability;
		}

		public double getProbability() {
			return _probability;
		}

		public PrivacyLevel getLevel() {
			return _level;
		}
	}
}
