package de.tudarmstadt.maki.simonstrator.api.component.relevance.path;

import java.util.ArrayList;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkRoute;

public class StatisticalRoadNetworkRoute {
    private double probability;
    private long duration;
	private RoadNetworkRoute path;

	public StatisticalRoadNetworkRoute(RoadNetworkRoute pPath, double pProbability, long pDuration) {
		path = pPath;

        probability = pProbability;
        duration = pDuration;
    }

    public StatisticalRoadNetworkRoute(RoadNetworkEdge pEdge, double pProbability, long pDuration) {
        List<RoadNetworkEdge> edges = new ArrayList<>();
        edges.add(pEdge);
        path = new RoadNetworkRoute(edges);

        probability = pProbability;
        duration = pDuration;
    }

    public long getDuration() {
        return duration;
    }

    public double getProbability() {
        return probability;
    }

    public RoadNetworkEdge getCurrentEdge() {
		return path.getStart();
    }

	public RoadNetworkRoute getPath() {
		return path;
    }

    public StatisticalRoadNetworkRoute createPath(RoadNetworkEdge pCurrentEdge, double pProbability, long pTravelTime) {
		ArrayList<RoadNetworkEdge> newEdges = new ArrayList<>(path.getRoute());
        newEdges.add(0, pCurrentEdge);
		return new StatisticalRoadNetworkRoute(new RoadNetworkRoute(newEdges), getProbability() * pProbability,
				getDuration() + pTravelTime);
    }

	public static StatisticalRoadNetworkRoute append(StatisticalRoadNetworkRoute pRoute,
			StatisticalRoadNetworkRoute pToBeAppended) {
		if (pRoute.getPath().getDestination().equals(pToBeAppended.getPath().getStart())) {
			List<RoadNetworkEdge> path = new ArrayList<>(pRoute.getPath().getRoute());
			for (int i = 1; i < pToBeAppended.getPath().getRoute().size(); i++) {
				path.add(pToBeAppended.getPath().getRoute().get(i));
			}

			return new StatisticalRoadNetworkRoute(new RoadNetworkRoute(path),
					pRoute.getProbability() * pToBeAppended.getProbability(),
					pRoute.getDuration() + pToBeAppended.getDuration());
		}
		throw new AssertionError("The two routes are not connected!");
	}

}
