/*
 * Copyright (c) 2005-2010 KOM � Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.relevance.vehicular.impl;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.PointInformation;

public class SimpleQoIBasedImpactFunction<T extends PointInformation> extends AbstractQoIBasedImpactFunction<T> {

	public SimpleQoIBasedImpactFunction(List<T> pInformation, double pAccuracy, int pCacheSize) {
		super(pInformation, pAccuracy, pCacheSize);
	}

	@Override
	public double calculateImpact(T pInformation) {
		return accuracy * super.calculateImpact(pInformation);
	}

	@Override
	protected double determineB(Object pUnused, Object pPossibleValue, double pChangeRate, double pRate,
			double pErrorProbability, double pNumberOfMessages, double pCostWrongKeep, double pCostWrongChange) {
		return determineB(pPossibleValue, pChangeRate, pRate, pErrorProbability, pNumberOfMessages, pCostWrongKeep,
				pCostWrongChange, 1);
	}

	protected double determineB(Object pPossibleValue, double pChangeRate, double pRate, double pErrorProbability,
			double pNumberOfMessages, double pCostWrongKeep, double pCostWrongChange, int reversed) {
		if (pErrorProbability == 0 || pErrorProbability == 1 || pErrorProbability == 0.5) {
			return Double.NaN;
		}

		double b;

		int optimalAmount = getOptimalMessageAmountForSwitch(pChangeRate, pErrorProbability, pCostWrongKeep,
				pCostWrongChange);

		if (optimalAmount == 1) {
			return Double.NEGATIVE_INFINITY;
		}

		boolean first = true;

		double leftSide;
		double rightSide;

		double step = 5;

		if (pErrorProbability < 0.5) {
			b = -2 * step * reversed;
		} else {
			b = 2 * step * reversed;
		}

		int similar = 0;
		double lastDifference = -1;
		do {
			leftSide = calculateWeightingForOldState(optimalAmount, pRate, pErrorProbability, pNumberOfMessages, b);
			rightSide = calculateWeightingForNewState(optimalAmount, pRate, pErrorProbability, pNumberOfMessages, b);
			if (Math.abs(Math.round((rightSide - leftSide) * ACCURACY_FACTOR)) == lastDifference) {
				similar++;
			} else {
				lastDifference = Math.abs(Math.round((rightSide - leftSide) * ACCURACY_FACTOR));
				similar = 0;
			}

			if (Double.isNaN(leftSide) || Double.isNaN(rightSide) || similar > 100) {
				if (reversed != -1) {
					double determineB = determineB(pPossibleValue, pChangeRate, pRate, pErrorProbability,
							pNumberOfMessages, pCostWrongKeep, pCostWrongChange, -1);
					if (!Double.isNaN(determineB)) {
						return determineB;
					} else {
						return b;
					}
				} else {
					return Double.NaN;
				}
			}

			leftSide = Math.round(leftSide * ACCURACY_FACTOR);
			rightSide = Math.round(rightSide * ACCURACY_FACTOR);

			if (leftSide > rightSide) {
				if (b < 0) {
					b -= step;
					if (!first) {
						step *= 0.5;
					}
				} else {
					b -= step;
					step *= 0.5;
					first = false;
				}
			} else if (leftSide < rightSide) {
				if (b > 0) {
					b += step;
					if (!first) {
						step *= 0.5;
					}
				} else {
					b += step;
					step *= 0.5;
					first = false;
				}
			} else {
				break;
			}
		} while (true);

		return b;
	}

	public double calculateWeightingForOldState(int optimalMessageAmount, double rate, double errorProbability,
			double pNumberOfMessages, double b) {
		double impact = 0;
		for (int a = optimalMessageAmount; a < pNumberOfMessages; a++) {
			impact += calculateImpact(errorProbability, pNumberOfMessages, pNumberOfMessages - a, b);
		}
		return impact;
	}

	public double calculateWeightingForNewState(int optimalMessageAmount, double rate, double errorProbability,
			double numberOfMessages, double b) {
		double impact = 0;
		for (int a = 0; a < optimalMessageAmount; a++) {
			impact += calculateImpact(errorProbability, numberOfMessages, numberOfMessages - a, b);
		}
		return impact;
	}

}
