/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.Environment;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.EnvironmentProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.plugin.EnvironmentSensorPlugin;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 27.02.2018
 *
 */
public class ExtendableEnvironmentSensor implements EnvironmentSensor {

	private Host _host;
	private List<EnvironmentSensorPlugin> _plugins;

	public ExtendableEnvironmentSensor(Host pHost, List<EnvironmentSensorPlugin> pPlugins) {
		_host = pHost;
		_plugins = pPlugins;
	}

	@Override
	public Host getHost() {
		return _host;
	}

	@Override
	public void initialize() {
	}

	@Override
	public void shutdown() {
	}

	public List<EnvironmentSensorPlugin> getPlugins() {
		return Collections.unmodifiableList(_plugins);
	}

	@Override
	public Environment getEnvironment() {
		return getEnvironment(new Class[0]);
	}

	@Override
	public Environment getEnvironment(Class... pAllowedClasses) {
		if (pAllowedClasses.length == 0) {
			List<EnvironmentProperty> properties = new ArrayList<>();
			for (EnvironmentSensorPlugin plugin : _plugins) {
				properties.addAll(plugin.getEnvironmentProperties());
			}
			return new Environment(properties);
		} else {
			List<EnvironmentProperty> properties = new ArrayList<>();
			for (EnvironmentSensorPlugin plugin : _plugins) {
				boolean allowed = false;
				for (int i = 0; i < pAllowedClasses.length; i++) {
					if (pAllowedClasses[i].isAssignableFrom(plugin.getSupportedClass())) {
						allowed = true;
					}
				}
				if (allowed) {
					properties.addAll(plugin.getEnvironmentProperties());
				}
			}
			return new Environment(properties);
		}
	}

}
