/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.costs;

import java.util.ArrayList;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.pubsub.Notification;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.BumpProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.FogProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.HazardProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.JamProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.LocationBasedEnvironmentProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RainProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.RoadProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.TrafficSignProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties.VehicleProperty;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.EnvironmentInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.RoadInformation;

public class DefaultVehicularPropertyImpactEstimator implements VehicularPropertyImpactEstimator {

	@Override
	public double calculateImpactIfKnown(Class<? extends RoadProperty> pProperty) {
		if (pProperty.equals(JamProperty.class)) {
			return Math.pow(10, 2);
		} else if (pProperty.equals(RainProperty.class)) {
			return 1;
		} else if (pProperty.equals(FogProperty.class)) {
			return 1;
		} else if (pProperty.equals(HazardProperty.class)) {
			return Math.pow(10, 3);
		} else if (pProperty.equals(TrafficSignProperty.class)) {
			return 10;
		} else if (pProperty.equals(VehicleProperty.class)) {
			return 0;
		}
		return 10;
	}

	@Override
	public List<Double> getImpactLevels() {
		List<Double> levels = new ArrayList<>();
		levels.add(1d);
		levels.add(10d);
		levels.add(100d);
		levels.add(1000d);
		return levels;
	}

	@Override
	public double calculateImpactIfKnown(Notification pNotification) {
		EnvironmentInformation<? extends LocationBasedEnvironmentProperty> environmentInformation = EnvironmentInformation
				.createFromNotification(pNotification);

		if (environmentInformation instanceof RoadInformation) {
			return calculateImpactIfKnown(((RoadInformation) environmentInformation).getValue());
		}

		throw new AssertionError("Cost calculation only valid for road information!");
	}

	@Override
	public double calculateImpactIfUnknown(Notification pNotification) {
		EnvironmentInformation<? extends LocationBasedEnvironmentProperty> environmentInformation = EnvironmentInformation
				.createFromNotification(pNotification);

		if (environmentInformation instanceof RoadInformation) {
			return calculateImpactIfUnknown(((RoadInformation) environmentInformation).getValue());
		}

		throw new AssertionError("Cost calculation only valid for road information!");
	}

	@Override
	public double calculateImpactIfUnknown(Class<? extends RoadProperty> pProperty) {
		if (pProperty.equals(JamProperty.class)) {
			return Math.pow(10, 4);
		} else if (pProperty.equals(BumpProperty.class)) {
			return Math.pow(10, -2);
		} else if (pProperty.equals(HazardProperty.class)) {
			return Math.pow(10, 6);
		}
		return 1;
	}

}
