/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties;

import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 27.02.2018
 *
 */
public class JamProperty extends AbstractRoadProperty {

	private final boolean _jammed;
	private double _averageSpeed = -1;
	
	public JamProperty(Location pLocation, RoadNetworkEdge pEdge, boolean pJammed) {
		this(pLocation, pEdge, pJammed, -1);

		if (_jammed) {
			_averageSpeed = 15;
		} else {
			_averageSpeed = -1;
		}
	}

	public JamProperty(Location pLocation, RoadNetworkEdge pEdge, boolean pJammed, double pAverageSpeed) {
		super(pLocation, pEdge);

		_jammed = pJammed;
		_averageSpeed = pAverageSpeed;
	}

	public void resetDetectionDate() {
		setDetectionDate(Time.getCurrentTime());
	}

	public boolean isJammed() {
		return JamProperty.isJammed(getEdge(), getAverageSpeed());
	}

	public double getAverageSpeed() {
		return _averageSpeed;
	}

	public void setAverageSpeed(double pAverageSpeed) {
		_averageSpeed = pAverageSpeed;
	}

	@Override
	public RoadProperty getDefaultProperty() {
		return new JamProperty(getLocation(), getEdge(), false, -1);
	}

	public static boolean isJammed(RoadNetworkEdge pEdge, double pSpeed) {
		if (pEdge != null) {
			double originalMaxSpeed = pEdge.getOriginalMaxSpeed();

			if (pSpeed < originalMaxSpeed * 0.675) {
				return true;
			}
		}
		return false;
	}

	@Override
	public Object getValue() {
		return getAverageSpeed();
	}

	@Override
	public JamProperty clone() {
		JamProperty jamProperty = new JamProperty(getLocation(), getEdge(), _jammed, _averageSpeed);
		jamProperty.setDuration(getDuration());
		jamProperty.setDetectionDate(getDetectionDate());
		return jamProperty;
	}

}
