/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.properties;

import de.tudarmstadt.maki.simonstrator.api.component.privacy.PrivacyLevel;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

/**
 * @author Tobias Meuser (tobias.meuser@kom.tu-darmstadt.de)
 * @version 1.0 at 27.02.2018
 *
 */
public class VehicleProperty extends AbstractRoadProperty {

	private final long _id;
	private final double _speed;
	private final double _length;
	
	private PrivacyLevel _privacy;

	public VehicleProperty(long pId, Location pLocation, RoadNetworkEdge pEdge, double pSpeed, double pLength) {
		this(pId, pLocation, pEdge, pSpeed, pLength, PrivacyLevel.NO_PRIVACY);
	}

	public VehicleProperty(long pId, Location pLocation, RoadNetworkEdge pEdge, double pSpeed, double pLength,
			PrivacyLevel pPrivacyLevel) {
		super(pLocation, pEdge);

		_id = pId;
		_speed = pSpeed;
		_length = pLength;
		_privacy = pPrivacyLevel;
	}

	public long getId() {
		return _id;
	}

	public double getSpeed() {
		return _speed;
	}

	@Override
	public Object getValue() {
		return getSpeed();
	}

	public double getLength() {
		return _length;
	}

	public PrivacyLevel getPrivacy() {
		return _privacy;
	}

	@Override
	public RoadProperty getDefaultProperty() {
		return null;
	}

	@Override
	public VehicleProperty clone() {
		return new VehicleProperty(_id, getLocation(), getEdge(), _speed, _length, _privacy);
	}

}
