/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.generator.lifetime;

import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

public class DeterministicExponentialLifetimeDistribution implements LifetimeDistribution {
	private long _averageEventDuration;
	private double _existProbability;

	private double _percentile;

	@XMLConfigurableConstructor({ "duration", "percentile" })
	public DeterministicExponentialLifetimeDistribution(long pEventDuration, double pPercentile) {
		_averageEventDuration = pEventDuration;

		_existProbability = 1 - 1 / (double) (pEventDuration / Time.SECOND);

		_percentile = pPercentile;
	}

	@XMLConfigurableConstructor({ "duration" })
	public DeterministicExponentialLifetimeDistribution(long pEventDuration) {
		_averageEventDuration = pEventDuration;

		_existProbability = 1 - 1 / (double) (pEventDuration / Time.SECOND);

		_percentile = 0.5;
	}

	@Override
	public long getLifetime() {
		return (long) Math.ceil(Math.log(_percentile) / Math.log(_existProbability)) * Time.SECOND;
	}

	@Override
	public long getAverageLifetime() {
		return _averageEventDuration;
	}

	@Override
	public double getProbabilityForLifetime(long pLifetime) {
		return Math.pow(_existProbability, pLifetime / Time.SECOND);
	}
}
