/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.route;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;

/**
 * Basic interface for a route object (unifying SUMO and OSM-based models).
 * 
 * @author Bjoern Richerzhagen
 *
 */
public interface Route {

	/**
	 * Target location of this route.
	 * 
	 * @return
	 */
	public Location getTargetLocation();

	/**
	 * True, if the Route contains segment information.
	 * 
	 * @return
	 */
	public boolean hasSegmentInformation();

	/**
	 * Segment we are currently on.
	 * 
	 * @return
	 */
	public RouteSegment getCurrentSegment();

	/**
	 * List of segments that are still ahead of us (this list INCLUDES the
	 * current segment!)
	 * 
	 * @return segments ahead, in order (the next segment is the first entry in
	 *         the list).
	 */
	public List<RouteSegment> getSegmentsAhead();

	/**
	 * List of segments we already left behind (this list does NOT include the
	 * current segment).
	 * 
	 * @return segments left behind, in order (the segment we just left behind
	 *         is the LAST entry in the list).
	 */
	public List<RouteSegment> getSegmentsBehind();

	/**
	 * A route segment (path segment), basically (part of) a road. This is
	 * assumed to have a unique string ID.
	 * 
	 * @author Bjoern Richerzhagen
	 *
	 */
	public interface RouteSegment {

		/**
		 * Unique ID of this route segment
		 * 
		 * @return
		 */
		public String getSegmentId();

		/**
		 * Route points of this segment (at least the START and END location,
		 * but may provide additional intermediate points).
		 * 
		 * @return
		 */
		public List<Location> getSegmentLocations();

	}

}
