/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.caching;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.common.graph.INodeID;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.overlay.NodeInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.caching.decision.CacheDecisionStrategy;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.PointInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

public interface CachingComponent extends HostComponent, NodeInformation {
	static final String DECISION_STRATEGY = "DecisionStrategy";

	<T extends PointInformation> List<T> getCacheEntries(Class<T> pCacheEntryClass);

	<T extends PointInformation> List<T> getDecidedCacheEntries(Class<T> pCacheEntryClass);

	<T extends PointInformation> List<T> getDecidedCacheEntries(Class<T> pCacheEntryClass, RoadNetworkEdge pEdge);

	<T extends PointInformation> List<T> getDecidedCacheEntries(Class<T> pCacheEntryClass, Class<?> pCacheValueClass,
			RoadNetworkEdge pEdge);

	<T extends PointInformation> List<T> getDecidedCacheEntries(Class<T> pCacheEntryClass, Class<?> pCacheValueClass,
			RoadNetworkEdge pEdge, INodeID pWithoutID);

	<T extends PointInformation> boolean containsEntry(T pCacheEntry);

	<T extends PointInformation> void storeCacheEntry(T pCacheEntry);

	void clear();

	default <T extends PointInformation> void partialClear(Class<T> pCacheEntryClass, RoadNetworkEdge pEdge) {
		partialClear(pCacheEntryClass, null, pEdge);
	}

	default <T extends PointInformation> void partialClear(Class<T> pCacheEntryClass) {
		partialClear(pCacheEntryClass, null);
	}

	<T extends PointInformation> void partialClear(Class<T> pCacheEntryClass, Class<?> pCacheValueClass,
			RoadNetworkEdge pEdge);

	void performDecisionTransition(Class<? extends Object> pInformationClass,
			Class<? extends CacheDecisionStrategy> pCacheDecisionStrategy);

	void adjustCacheSizePerEntry(Class<? extends Object> pInformationClass, int pMaxCacheSizePerEntry);

	void setMaxSize(int pMaxCacheSize);

	void registerCacheStateListener(CacheStateListener pListener);

}
